Fix SF bug #689659, 64-bit int and long hash keys incompatible

On a 64-bit machine, a dictionary could contain duplicate int/long keys
if the value was > 2**32.
diff --git a/Lib/test/test_types.py b/Lib/test/test_types.py
index 449d0dd..d571a02 100644
--- a/Lib/test/test_types.py
+++ b/Lib/test/test_types.py
@@ -639,6 +639,14 @@
 except KeyError: pass
 else: raise TestFailed, "{}.pop(k) doesn't raise KeyError when dictionary is empty"
 
+# verify longs/ints get same value when key > 32 bits (for 64-bit archs)
+# see SF bug #689659
+x = 4503599627370496L
+y = 4503599627370496
+h = {x: 'anything', y: 'something else'} 
+if h[x] != h[y]:
+    raise TestFailed, "long/int key should match"
+
 d[1] = 1
 try:
     for i in d:
diff --git a/Misc/NEWS b/Misc/NEWS
index 8d5a669..c123689 100644
--- a/Misc/NEWS
+++ b/Misc/NEWS
@@ -12,7 +12,8 @@
 Core and builtins
 -----------------
 
-TBD
+- On 64-bit systems, a dictionary could contain duplicate long/int keys
+  if the key value was larger than 2**32.  See SF bug #689659.
 
 Extension modules
 -----------------
diff --git a/Objects/longobject.c b/Objects/longobject.c
index c2d6ea7..446affb 100644
--- a/Objects/longobject.c
+++ b/Objects/longobject.c
@@ -1490,11 +1490,13 @@
 		sign = -1;
 		i = -(i);
 	}
+#define LONG_BIT_SHIFT	(8*sizeof(long) - SHIFT)
 	while (--i >= 0) {
-		/* Force a 32-bit circular shift */
-		x = ((x << SHIFT) & ~MASK) | ((x >> (32-SHIFT)) & MASK);
+		/* Force a native long #-bits (32 or 64) circular shift */
+		x = ((x << SHIFT) & ~MASK) | ((x >> LONG_BIT_SHIFT) & MASK);
 		x += v->ob_digit[i];
 	}
+#undef LONG_BIT_SHIFT
 	x = x * sign;
 	if (x == -1)
 		x = -2;